/**************************************************************************/
/* FILE NAME: main.c           COPYRIGHT (c) Freescale Semiconductor 2004 */ 
/* VERSION:   1.0                                 All Rights Reserved     */
/*                                                                        */
/* DESCRIPTION: eTPU Stepper Motor Demo                                   */
/*========================================================================*/
/* REV      AUTHOR      DATE        DESCRIPTION OF CHANGE                 */
/* ---   -----------  ----------    ---------------------                 */
/* 1.0   M. Brejl     01/Jul/04     First release.                        */
/* 2.0   M. Brejl     12/Dec/04     Updated for SM API 2.0.                        */
/*                                                                        */
/**************************************************************************/

/*******************************************************************************
 * Includes
 *******************************************************************************/
#include "m523xevb.h"    /* MCF523xEVB definitions */

#include "typedefs.h"
#include "mcf523x_vars.h"
#include "etpu_util.h"   /* useful eTPU utility routines */

#include "etpu_sm.h"     /* eTPU SM API */
#include "3phase_sm_etpu_gct.h" /* eTPU configuration created by eTPU GCT */

#include "fmaster.h"     /* FreeMaster support */


/*******************************************************************************
 * Function prototypes
 *******************************************************************************/
void ApplicationStateMachine( void);
void ApplicationStatusLed( void);
void iniFmasterUart(void);
void iniPit3(void);

/*******************************************************************************
 * Variables
 *******************************************************************************/

/* Application States */
#define APP_STATE_OFF        0
#define APP_STATE_ENABLE     1
#define APP_STATE_ON         2
#define APP_STATE_DISABLE    3
#define APP_STATE_FAULT      4

/* ON/OFF Switch */
#define SWITCH_STATUS_OFF    0
#define SWITCH_STATUS_ON     1

/* Application Buttons And Status LED */
volatile uint16 TimeoutLED = 200;
volatile uint16 ApplicationState = APP_STATE_FAULT;
volatile uint16 SwitchStatus;

/* Stepper Motor */
volatile int32  StartFrequency = 2;  /* [Hz] */
volatile int32  SlewFrequency = 20;  /* [Hz] */

volatile int32  DesiredPosition = 0;
volatile int32  LastDesiredPosition = 0;
volatile int32  CurrentPosition;
volatile uint8  Flags;

extern etpu_a_tcr1_freq;

/* if Revision A of M523xEVB is used, then uncomment the following line */
//#define M523xEVB_REV_A

/*******************************************************************************
 * Main
 *******************************************************************************/
void main (void)
{
  /* Initialize UART for FreeMaster */
  iniFmasterUart();

  /* Initialize FreeMaster */
  fmasterInit();

  /* Initialize eTPU */
  my_system_etpu_init();

  /* Initialize Interrupt Timer 3 */
  iniPit3();

  /* Start eTPU */
  my_system_etpu_start();

  /* Idle */
  while(1) ;
}


/*******************************************************************************
 * Application state machine
 *******************************************************************************/
void ApplicationStateMachine( void)
{

  switch (ApplicationState)
  {
    case APP_STATE_OFF:
      if (SwitchStatus == SWITCH_STATUS_ON)
      {
        ApplicationState = APP_STATE_ENABLE;
      }
      
      break;
    case APP_STATE_ENABLE:
      // Enable SM
      fs_etpu_sm_enable(SM0_A_MASTER,   /* engine: A; channel: 8 */
                        FS_ETPU_SM_3PHASE_HALF_STEP);   /* configuration: 3-phase half-step */

      ApplicationState = APP_STATE_ON;

      break;
    case APP_STATE_ON:
      // Set Desired Position
      if (DesiredPosition != LastDesiredPosition)
      {
        fs_etpu_sm_set_dp(SM0_A_MASTER, DesiredPosition);
        LastDesiredPosition = DesiredPosition;
      }
            
      if (SwitchStatus == SWITCH_STATUS_OFF)
      {
        ApplicationState = APP_STATE_DISABLE;
      }
      
      break;
    case APP_STATE_DISABLE:
      // Disable SM
      fs_etpu_sm_disable(SM0_A_MASTER,  /* engine: A; channel: 8 */
                         FS_ETPU_SM_DISABLE_LOW);   /* disable polarity: low */
            
      ApplicationState = APP_STATE_OFF;
      
      break;
    case APP_STATE_FAULT:
      // Disable SM
      fs_etpu_sm_disable(SM0_A_MASTER,  /* engine: A; channel: 8 */
                         FS_ETPU_SM_DISABLE_LOW);   /* disable polarity: low */
            
      if (!SwitchStatus)
      {
        ApplicationState = APP_STATE_DISABLE;
      }

      break;
  }
}


/*******************************************************************************
 * Service of Status LED
 *******************************************************************************/
void ApplicationStatusLed( void)
{

  if (TimeoutLED-- == 0)
  {
    switch (ApplicationState)
    {
      case APP_STATE_OFF:
      case APP_STATE_DISABLE:
      case APP_STATE_ENABLE:
        TimeoutLED = 200;
        // Toggles the LED 
        MCF_GPIO_PODR_BUSCTL ^= MCF_GPIO_PODR_BUSCTL_PODR_BUSCTL0;
        break;
      case APP_STATE_ON:
        MCF_GPIO_PODR_BUSCTL |= MCF_GPIO_PODR_BUSCTL_PODR_BUSCTL0;
        TimeoutLED = 50;
        break;
      case APP_STATE_FAULT:
        TimeoutLED = 50;
        // Toggles the LED 
        MCF_GPIO_PODR_BUSCTL ^= MCF_GPIO_PODR_BUSCTL_PODR_BUSCTL0;
        break;
    }
  }
}


/*******************************************************************************
 * Interrupt service routine PIT3
 *******************************************************************************/
__declspec(interrupt) 
void timer3_isr(void)
{
  
#ifdef M523xEVB_REV_A
  SwitchStatus = (MCF_GPIO_PPDSDR_BUSCTL & MCF_GPIO_PPDSDR_BUSCTL_PPDSDR_BUSCTL5) > 0;
#else
  SwitchStatus = (MCF_GPIO_PPDSDR_BUSCTL & MCF_GPIO_PPDSDR_BUSCTL_PPDSDR_BUSCTL2) > 0;
#endif

  /* Stepper Motor */
  CurrentPosition = fs_etpu_sm_get_cp(SM0_A_MASTER);
  Flags = fs_etpu_sm_get_flags(SM0_A_MASTER);

  /* State Machine */
  ApplicationStateMachine();
  
  /* Led */
  ApplicationStatusLed();

  MCF_PIT_PCSR3 |= MCF_PIT_PCSR_PIF;  /* Erase PIT3 overflow flag */
}   
  

/*******************************************************************************
 * Initialization of interrupts and GPIO pins
 *******************************************************************************/
void iniPit3(void)
{

  MCF_INTC0_ICR39 = 0x9;
  //"Mask All" Disabled by IniPcmasterUart  
  MCF_INTC0_IMRH &= (~MCF_INTC0_IPRH_INT39);

  // Assigns TIP and TEA as GPIO pins
  MCF_GPIO_PAR_BUSCTL &= ~(MCF_GPIO_PAR_BUSCTL_PAR_TIP_TEA | 
                           MCF_GPIO_PAR_BUSCTL_PAR_TEA_TEA |
                           MCF_GPIO_PAR_BUSCTL_PAR_TSIZ0 | 
                           MCF_GPIO_PAR_BUSCTL_PAR_TSIZ1); 

  // Assigns TIP as GPIO output
  MCF_GPIO_PDDR_BUSCTL |= MCF_GPIO_PDDR_BUSCTL_PDDR_BUSCTL0;

  // Assigns TEA as GPIO input
  MCF_GPIO_PDDR_BUSCTL &= ~(MCF_GPIO_PDDR_BUSCTL_PDDR_BUSCTL5 | 
                            MCF_GPIO_PDDR_BUSCTL_PDDR_BUSCTL2 |
                            MCF_GPIO_PDDR_BUSCTL_PDDR_BUSCTL3);

  // Sets the LED off
  MCF_GPIO_PODR_BUSCTL &= (~MCF_GPIO_PODR_BUSCTL_PODR_BUSCTL0);
  
  // configure PIT - Prescaller 4
  MCF_PIT_PCSR3 = (0x0100 | MCF_PIT_PCSR_PIE | MCF_PIT_PCSR_EN);

  MCF_PIT_PMR3 = 18750;

  // configure Interrupt Controller
  MCF_PIT_PCSR3 |= MCF_PIT_PCSR_PIF;    
}


/********************************************************************/


/*********************************************************************
 *
 * Copyright:
 *  Freescale Semiconductor, INC. All Rights Reserved.
 *  You are hereby granted a copyright license to use, modify, and
 *  distribute the SOFTWARE so long as this entire notice is
 *  retained without alteration in any modified and/or redistributed
 *  versions, and that such modified versions are clearly identified
 *  as such. No licenses are granted by implication, estoppel or
 *  otherwise under any patents or trademarks of Freescale
 *  Semiconductor, Inc. This software is provided on an "AS IS"
 *  basis and without warranty.
 *
 *  To the maximum extent permitted by applicable law, Freescale
 *  Semiconductor DISCLAIMS ALL WARRANTIES WHETHER EXPRESS OR IMPLIED,
 *  INCLUDING IMPLIED WARRANTIES OF MERCHANTABILITY OR FITNESS FOR A
 *  PARTICULAR PURPOSE AND ANY WARRANTY AGAINST INFRINGEMENT WITH
 *  REGARD TO THE SOFTWARE (INCLUDING ANY MODIFIED VERSIONS THEREOF)
 *  AND ANY ACCOMPANYING WRITTEN MATERIALS.
 *
 *  To the maximum extent permitted by applicable law, IN NO EVENT
 *  SHALL Freescale Semiconductor BE LIABLE FOR ANY DAMAGES WHATSOEVER
 *  (INCLUDING WITHOUT LIMITATION, DAMAGES FOR LOSS OF BUSINESS PROFITS,
 *  BUSINESS INTERRUPTION, LOSS OF BUSINESS INFORMATION, OR OTHER
 *  PECUNIARY LOSS) ARISING OF THE USE OR INABILITY TO USE THE SOFTWARE.
 *
 *  Freescale Semiconductor assumes no responsibility for the
 *  maintenance and support of this software
 ********************************************************************/
